@@ -10,6 +10,8 @@
 #include "kpublictransport_export.h"
 
 #include "datatypes.h"
+#include "feature.h"
+#include "load.h"
 
 namespace KPublicTransport {
 
@@ -67,7 +69,9 @@ class KPUBLICTRANSPORT_EXPORT VehicleSection
      */
     KPUBLICTRANSPORT_PROPERTY(Classes, classes, setClasses)
 
-    /** Amenities or other features available in a vehicle section. */
+    /** Amenities or other features available in a vehicle section.
+     *  @deprecated Use KPublicTransport::Feature instead.
+     */
     enum Feature {
         NoFeatures = 0,
         AirConditioning = 1, ///< vehicle section is air conditioned
@@ -76,17 +80,34 @@ class KPUBLICTRANSPORT_EXPORT VehicleSection
         WheelchairAccessible = 8, ///< wheelchair access supported
         SilentArea = 16, ///< wishful thinking usually
         BikeStorage = 32, ///< vehicle section contains space for bikes
-        // TODO there's a few more we get from DB
     };
     Q_DECLARE_FLAGS(Features, Feature)
     Q_FLAG(Features)
     /** Features available in this vehicle section.
      *  Can be more than one.
+     *  @deprecated use sectionFeatures instead
+     */
+    Q_PROPERTY(Features features READ features WRITE setFeatures STORED false)
+    [[nodiscard]] [[deprecated("use sectionFeatures")]] Features features() const;
+    [[deprecated("use setSectionFeatures")]] void setFeatures(Features features);
+
+    /** Feature flag as a variant list, for consumption in QML.
+     *  @deprecated use sectionFeatures instead
      */
-    KPUBLICTRANSPORT_PROPERTY(Features, features, setFeatures)
-    /** Feature flag as a variant list, for consumption in QML. */
     Q_PROPERTY(QVariantList featureList READ featureList STORED false)
 
+    /** Features of this section, for consumption by QML. */
+    Q_PROPERTY(std::vector<KPublicTransport::Feature> sectionFeatures READ sectionFeatures STORED false)
+
+    [[nodiscard]] const std::vector<KPublicTransport::Feature>& sectionFeatures() const;
+    [[nodiscard]] std::vector<KPublicTransport::Feature>&& takeSectionFeatures();
+    void setSectionFeatures(std::vector<KPublicTransport::Feature> &&features);
+
+    /** Returns the first feature of type @p type.
+     *  Returns an invalid feature of no feature of that type exists.
+     */
+    Q_INVOKABLE [[nodiscard]] KPublicTransport::Feature feature(KPublicTransport::Feature::Type type) const;
+
     /** Number of decks in this vehicle section. */
     KPUBLICTRANSPORT_PROPERTY(int, deckCount, setDeckCount)
 
@@ -107,28 +128,36 @@ class KPUBLICTRANSPORT_EXPORT VehicleSection
      */
     KPUBLICTRANSPORT_PROPERTY(Sides, connectedSides, setConnectedSides)
 
+    /** Distruption affecting this coach.
+     *  Disruption::NoService here means the coach is closed.
+     */
+    KPUBLICTRANSPORT_PROPERTY(KPublicTransport::Disruption::Effect, disruptionEffect, setDisruptionEffect)
+
+    /** Occupancy level for this coach. */
+    KPUBLICTRANSPORT_PROPERTY(KPublicTransport::Load::Category, load, setLoad)
+
     /** Name of the platform section(s) this coach is position in.
      *  This is primarily meant as a fallback when exact platform positions aren't available.
      */
     KPUBLICTRANSPORT_PROPERTY(QString, platformSectionName, setPlatformSectionName)
 
     /** Returns @c true if this vehicle section has a valid platform position set. */
-    bool hasPlatformPosition() const;
+    [[nodiscard]] bool hasPlatformPosition() const;
 
     /** Merge two VehicleSection instances. */
-    static VehicleSection merge(const VehicleSection &lhs, const VehicleSection &rhs);
+    [[nodiscard]] static VehicleSection merge(const VehicleSection &lhs, const VehicleSection &rhs);
 
     /** Serializes one vehicle section to JSON. */
-    static QJsonObject toJson(const VehicleSection &section);
+    [[nodiscard]] static QJsonObject toJson(const VehicleSection &section);
     /** Serializes a vector of vehicle sections to JSON. */
-    static QJsonArray toJson(const std::vector<VehicleSection> &sections);
+    [[nodiscard]] static QJsonArray toJson(const std::vector<VehicleSection> &sections);
     /** Deserialize an object from JSON. */
-    static VehicleSection fromJson(const QJsonObject &obj);
+    [[nodiscard]] static VehicleSection fromJson(const QJsonObject &obj);
     /** Deserialize a vector of vehicle sections from JSON. */
-    static std::vector<VehicleSection> fromJson(const QJsonArray &array);
+    [[nodiscard]] static std::vector<VehicleSection> fromJson(const QJsonArray &array);
 
 private:
-    QVariantList featureList() const;
+    [[nodiscard]] QVariantList featureList() const;
 };
 
 class VehiclePrivate;
@@ -168,46 +197,60 @@ class KPUBLICTRANSPORT_EXPORT Vehicle
      */
     Q_PROPERTY(float platformPositionEnd READ platformPositionEnd STORED false)
 
+    /** Features of this vehicle as a whole, not including the features of individual sections. */
+    Q_PROPERTY(std::vector<KPublicTransport::Feature> features READ features)
+
+    /** Features of the entire vehicle including a union of all features of the individual sections. */
+    Q_PROPERTY(std::vector<KPublicTransport::Feature> combinedFeatures READ combinedFeatures)
+
 public:
     /** Returns @c true if this object contains no information beyond the default values. */
-    bool isEmpty() const;
+    [[nodiscard]] bool isEmpty() const;
 
     /** The vehicle sections. */
-    const std::vector<VehicleSection>& sections() const;
+    [[nodiscard]] const std::vector<VehicleSection>& sections() const;
     /** Moves the vehicle sections out of this object. */
-    std::vector<VehicleSection>&& takeSections();
+    [[nodiscard]] std::vector<VehicleSection>&& takeSections();
     /** Sets the vehicle sections. */
     void setSections(std::vector<VehicleSection> &&sections);
     void setSections(const std::vector<VehicleSection> &sections);
 
-    float platformPositionBegin() const;
-    float platformPositionEnd() const;
+    [[nodiscard]] float platformPositionBegin() const;
+    [[nodiscard]] float platformPositionEnd() const;
 
     /** Returns the center position of the vehicle section named @p sectionName
      *  in relative platform coordinates.
      *  Useful for centering a view on a selected section for example.
      */
-    Q_INVOKABLE float platformPositionForSection(const QString &sectionName) const;
+    Q_INVOKABLE [[nodiscard]] float platformPositionForSection(const QString &sectionName) const;
 
     /** Checks whether all vehicle sections have platform positions set. */
-    bool hasPlatformPositions() const;
+    [[nodiscard]] bool hasPlatformPositions() const;
     /** Check whether all vehicle sections have platform section names set. */
-    bool hasPlatformSectionNames() const;
+    [[nodiscard]] bool hasPlatformSectionNames() const;
+
+    /** Vehicle features. */
+    [[nodiscard]] const std::vector<KPublicTransport::Feature>& features() const;
+    [[nodiscard]] std::vector<KPublicTransport::Feature>&& takeFeatures();
+    void setFeatures(std::vector<KPublicTransport::Feature> &&features);
+
+    /** Combined vehicle features. */
+    [[nodiscard]] std::vector<KPublicTransport::Feature> combinedFeatures() const;
 
     /** Merge two Vehicle instances. */
-    static Vehicle merge(const Vehicle &lhs, const Vehicle &rhs);
+    [[nodiscard]] static Vehicle merge(const Vehicle &lhs, const Vehicle &rhs);
 
     /** Serializes one vehicle object to JSON. */
-    static QJsonObject toJson(const Vehicle &vehicle);
+    [[nodiscard]] static QJsonObject toJson(const Vehicle &vehicle);
     /** Serializes multiple vehicle objects to JSON. */
-    static QJsonArray toJson(const std::vector<Vehicle> &vehicles);
+    [[nodiscard]] static QJsonArray toJson(const std::vector<Vehicle> &vehicles);
     /** Deserialize an object from JSON. */
-    static Vehicle fromJson(const QJsonObject &obj);
+    [[nodiscard]] static Vehicle fromJson(const QJsonObject &obj);
     /** Deserialize multiple objects from JSON. */
-    static std::vector<Vehicle> fromJson(const QJsonArray &array);
+    [[nodiscard]] static std::vector<Vehicle> fromJson(const QJsonArray &array);
 
 private:
-    QVariantList sectionsVariant() const;
+    [[nodiscard]] [[deprecated("use sectionFeatures instead")]] QVariantList sectionsVariant() const;
 };
 
 Q_DECLARE_OPERATORS_FOR_FLAGS(VehicleSection::Classes)
